@@ -19,8 +19,8 @@ from farcaster.models import *
 from farcaster.utils.stream_generator import stream_generator
 
 
-class MerkleApiClient:
-    """The MerkleApiClient class is a wrapper around the Farcaster API.
+class Warpcast:
+    """The Warpcast class is a wrapper around the Farcaster API.
     It also provides a number of helpful methods and utilities for interacting with the protocol.
     Pydantic models are used under the hood to validate the data returned from the API.
     """
@@ -167,7 +167,7 @@ class MerkleApiClient:
         self,
         cursor: NoneStr = None,
         limit: PositiveInt = 25,
-    ) -> EventsResult:
+    ) -> IterableEventsResult:
         """Get events for a given asset
 
         Args:
@@ -176,13 +176,17 @@ class MerkleApiClient:
                 to 25
 
         Returns:
-            EventsResult: Returns the EventsResult model
+            IterableEventsResult: Returns the EventsResult model with an optional cursor
         """
-        response = self._get(
-            "asset-events",
-            params={"cursor": cursor, "limit": limit},
+        response = AssetEventsGetResponse(
+            **self._get(
+                "asset-events",
+                params={"cursor": cursor, "limit": limit},
+            )
+        )
+        return IterableEventsResult(
+            events=response.result.events, cursor=getattr(response.next, "cursor", None)
         )
-        return AssetEventsGetResponse(**response).result
 
     def put_auth(self, auth_params: AuthParams) -> TokenResult:
         """Generate a custody bearer token and use it to generate an access token
@@ -221,7 +225,7 @@ class MerkleApiClient:
         cast_hash: str,
         cursor: NoneStr = None,
         limit: PositiveInt = 25,
-    ) -> ReactionsResult:
+    ) -> IterableReactionsResult:
         """Get the likes for a given cast
 
         Args:
@@ -230,13 +234,17 @@ class MerkleApiClient:
             limit (PositiveInt, optional): limit, defaults to 25
 
         Returns:
-            ReactionsResult: ReactionsResult model of likes
+            IterableReactionsResult: Model containing the likes with an optional cursor
         """
-        response = self._get(
-            "cast-likes",
-            params={"castHash": cast_hash, "cursor": cursor, "limit": limit},
+        response = CastReactionsGetResponse(
+            **self._get(
+                "cast-likes",
+                params={"castHash": cast_hash, "cursor": cursor, "limit": limit},
+            )
+        )
+        return IterableReactionsResult(
+            likes=response.result.likes, cursor=getattr(response.next, "cursor", None)
         )
-        return CastReactionsGetResponse(**response).result
 
     def like_cast(self, cast_hash: str) -> ReactionsPutResult:
         """Like a given cast
@@ -275,7 +283,7 @@ class MerkleApiClient:
         cast_hash: str,
         cursor: NoneStr = None,
         limit: PositiveInt = 25,
-    ) -> UsersResult:
+    ) -> IterableUsersResult:
         """Get the recasters for a given cast
 
         Args:
@@ -284,13 +292,17 @@ class MerkleApiClient:
             limit (PositiveInt, optional): limit, defaults to 25
 
         Returns:
-            UsersResult: Model containing the recasters
+            IterableUsersResult: Model containing the recasters with an optional cursor
         """
-        response = self._get(
-            "cast-recasters",
-            params={"castHash": cast_hash, "cursor": cursor, "limit": limit},
+        response = CastRecastersGetResponse(
+            **self._get(
+                "cast-recasters",
+                params={"castHash": cast_hash, "cursor": cursor, "limit": limit},
+            )
+        )
+        return IterableUsersResult(
+            users=response.result.users, cursor=getattr(response.next, "cursor", None)
         )
-        return CastRecastersGetResponse(**response).result
 
     def get_cast(
         self,
@@ -333,7 +345,7 @@ class MerkleApiClient:
         fid: int,
         cursor: NoneStr = None,
         limit: PositiveInt = 25,
-    ) -> CastsResult:
+    ) -> IterableCastsResult:
         """Get the casts for a given fid of a user
 
         Args:
@@ -342,25 +354,29 @@ class MerkleApiClient:
             limit (PositiveInt, optional): limit, defaults to 25
 
         Returns:
-            CastsResult: Model containing the casts
+            IterableCastsResult: Model containing the casts with an optional cursor
         """
-        response = self._get(
-            "casts",
-            params={"fid": fid, "cursor": cursor, "limit": limit},
+        response = CastsGetResponse(
+            **self._get(
+                "casts",
+                params={"fid": fid, "cursor": cursor, "limit": limit},
+            )
+        )
+        return IterableCastsResult(
+            casts=response.result.casts, cursor=getattr(response.next, "cursor", None)
         )
-        return CastsGetResponse(**response).result
 
     def post_cast(
         self,
         text: str,
-        embeds: Optional[List[AnyUrl]] = None,
+        embeds: Optional[List[str]] = None,
         parent: Optional[Parent] = None,
     ) -> CastContent:
         """Post a cast to Farcaster
 
         Args:
             text (str): text of the cast
-            embeds (Optional[List[AnyUrl]], optional): list of embeds, defaults to None
+            embeds (Optional[List[str]], optional): list of embeds, defaults to None
             parent (Optional[Parent], optional): parent of the cast, defaults to None
 
         Returns:
@@ -394,7 +410,7 @@ class MerkleApiClient:
         collection_id: str,
         cursor: NoneStr = None,
         limit: PositiveInt = 25,
-    ) -> UsersResult:
+    ) -> IterableUsersResult:
         """Get the owners of an OpenSea collection
 
         Args:
@@ -403,20 +419,28 @@ class MerkleApiClient:
             limit (PositiveInt, optional): limit, defaults to 25
 
         Returns:
-            UsersResult: model containing users
+            IterableUsersResult: model containing users with an optional cursor
         """
-        response = self._get(
-            "collection-owners",
-            params={"collectionId": collection_id, "cursor": cursor, "limit": limit},
+        response = CollectionOwnersGetResponse(
+            **self._get(
+                "collection-owners",
+                params={
+                    "collectionId": collection_id,
+                    "cursor": cursor,
+                    "limit": limit,
+                },
+            )
+        )
+        return IterableUsersResult(
+            users=response.result.users, cursor=getattr(response.next, "cursor", None)
         )
-        return CollectionOwnersGetResponse(**response).result
 
     def get_followers(
         self,
         fid: int,
         cursor: NoneStr = None,
         limit: PositiveInt = 25,
-    ) -> UsersResult:
+    ) -> IterableUsersResult:
         """Get the followers of a user
 
         Args:
@@ -424,21 +448,50 @@ class MerkleApiClient:
             cursor (NoneStr, optional): cursor, defaults to None
             limit (PositiveInt, optional): limit, defaults to 25
 
+        Returns:
+            IterableUsersResult: model containing users with an optional cursor
+        """
+        response = FollowersGetResponse(
+            **self._get(
+                "followers",
+                params={"fid": fid, "cursor": cursor, "limit": limit},
+            )
+        )
+        return IterableUsersResult(
+            users=response.result.users, cursor=getattr(response.next, "cursor", None)
+        )
+
+    def get_all_followers(self, fid: Optional[int] = None) -> UsersResult:
+        """Get all followers of a user by iterating through the next cursors
+        Args:
+            fid (int): Farcaster ID of the user
         Returns:
             UsersResult: model containing users
         """
-        response = self._get(
-            "followers",
-            params={"fid": fid, "cursor": cursor, "limit": limit},
-        )
-        return FollowersGetResponse(**response).result
+        users: List[ApiUser] = []
+        cursor = None
+        limit = 100
+        if fid is None:
+            fid = self.get_me().fid
+        while True:
+            response = self._get(
+                "followers",
+                params={"fid": fid, "cursor": cursor, "limit": limit},
+            )
+            response_model = FollowersGetResponse(**response)
+            if response_model.result.users:
+                users.extend(response_model.result.users)
+            if response_model.next is None:
+                break
+            cursor = response_model.next.cursor
+        return UsersResult(users=users)
 
     def get_following(
         self,
         fid: int,
         cursor: NoneStr = None,
         limit: PositiveInt = 25,
-    ) -> UsersResult:
+    ) -> IterableUsersResult:
         """Get the users a user is following
 
         Args:
@@ -447,13 +500,17 @@ class MerkleApiClient:
             limit (PositiveInt, optional): limit, defaults to 25
 
         Returns:
-            UsersResult: model containing users
+            IterableUsersResult: model containing users with an optional cursor
         """
-        response = self._get(
-            "following",
-            params={"fid": fid, "cursor": cursor, "limit": limit},
+        response = FollowingGetResponse(
+            **self._get(
+                "following",
+                params={"fid": fid, "cursor": cursor, "limit": limit},
+            )
+        )
+        return IterableUsersResult(
+            users=response.result.users, cursor=getattr(response.next, "cursor", None)
         )
-        return FollowingGetResponse(**response).result
 
     def get_all_following(self, fid: Optional[int] = None) -> UsersResult:
         """Get all the users a user is following by iterating through the next cursors
@@ -531,7 +588,7 @@ class MerkleApiClient:
         self,
         cursor: NoneStr = None,
         limit: PositiveInt = 25,
-    ) -> NotificationsResult:
+    ) -> IterableNotificationsResult:
         """Get mention and reply notifications
 
         Args:
@@ -539,13 +596,18 @@ class MerkleApiClient:
             limit (PositiveInt, optional): limit, defaults to 25
 
         Returns:
-            NotificationsResult: model containing notifications
+            IterableNotificationsResult: model containing notifications with an optional cursor
         """
-        response = self._get(
-            "mention-and-reply-notifications",
-            params={"cursor": cursor, "limit": limit},
+        response = MentionAndReplyNotificationsGetResponse(
+            **self._get(
+                "mention-and-reply-notifications",
+                params={"cursor": cursor, "limit": limit},
+            )
+        )
+        return IterableNotificationsResult(
+            notifications=response.result.notifications,
+            cursor=getattr(response.next, "cursor", None),
         )
-        return MentionAndReplyNotificationsGetResponse(**response).result
 
     def _recent_notifications_list(
         self,
@@ -678,7 +740,7 @@ class MerkleApiClient:
         owner_fid: int,
         cursor: NoneStr = None,
         limit: PositiveInt = 25,
-    ) -> CollectionsResult:
+    ) -> IterableCollectionsResult:
         """Get the collections of a user
 
         Args:
@@ -687,20 +749,25 @@ class MerkleApiClient:
             limit (PositiveInt, optional): limit, defaults to 25
 
         Returns:
-            CollectionsResult: model containing collections
+            IterableCollectionsResult: model containing collections with an optional cursor
         """
-        response = self._get(
-            "user-collections",
-            params={"ownerFid": owner_fid, "cursor": cursor, "limit": limit},
+        response = UserCollectionsGetResponse(
+            **self._get(
+                "user-collections",
+                params={"ownerFid": owner_fid, "cursor": cursor, "limit": limit},
+            )
+        )
+        return IterableCollectionsResult(
+            collections=response.result.collections,
+            cursor=getattr(response.next, "cursor", None),
         )
-        return UserCollectionsGetResponse(**response).result
 
     def get_verifications(
         self,
         fid: int,
         cursor: NoneStr = None,
         limit: PositiveInt = 25,
-    ) -> VerificationsResult:
+    ) -> IterableVerificationsResult:
         """Get the verifications of a user
 
         Args:
@@ -709,19 +776,24 @@ class MerkleApiClient:
             limit (PositiveInt, optional): limit, defaults to 25
 
         Returns:
-            VerificationsResult: model containing verifications
+            IterableVerificationsResult: model containing verifications with an optional cursor
         """
-        response = self._get(
-            "verifications",
-            params={"fid": fid, "cursor": cursor, "limit": limit},
+        response = VerificationsGetResponse(
+            **self._get(
+                "verifications",
+                params={"fid": fid, "cursor": cursor, "limit": limit},
+            )
+        )
+        return IterableVerificationsResult(
+            verifications=response.result.verifications,
+            cursor=getattr(response.next, "cursor", None),
         )
-        return VerificationsGetResponse(**response).result
 
     def get_recent_users(
         self,
         cursor: NoneStr = None,
         limit: PositiveInt = 25,
-    ) -> UsersResult:
+    ) -> IterableUsersResult:
         """Get recent users
 
         Args:
@@ -729,13 +801,17 @@ class MerkleApiClient:
             limit (PositiveInt, optional): limit, defaults to 25
 
         Returns:
-            UsersResult: model containing users
+            IterableUsersResult: model containing users with an optional cursor
         """
-        response = self._get(
-            "recent-users",
-            params={"cursor": cursor, "limit": limit},
+        response = UsersGetResponse(
+            **self._get(
+                "recent-users",
+                params={"cursor": cursor, "limit": limit},
+            )
+        )
+        return IterableUsersResult(
+            users=response.result.users, cursor=getattr(response.next, "cursor", None)
         )
-        return UsersGetResponse(**response).result
 
     def _recent_users_list(
         self,
@@ -802,7 +878,7 @@ class MerkleApiClient:
         fid: int,
         cursor: NoneStr = None,
         limit: PositiveInt = 25,
-    ) -> Likes:
+    ) -> IterableLikes:
         """Get the likes of a user
 
         Args:
@@ -811,19 +887,23 @@ class MerkleApiClient:
             limit (PositiveInt, optional): limit, defaults to 25
 
         Returns:
-            Likes: model containing likes
+            IterableLikes: model containing likes with an optional cursor
         """
-        response = self._get(
-            "user-cast-likes",
-            params={"fid": fid, "cursor": cursor, "limit": limit},
+        response = UserCastLikesGetResponse(
+            **self._get(
+                "user-cast-likes",
+                params={"fid": fid, "cursor": cursor, "limit": limit},
+            )
+        )
+        return IterableLikes(
+            likes=response.result.likes, cursor=getattr(response.next, "cursor", None)
         )
-        return UserCastLikesGetResponse(**response).result
 
     def get_recent_casts(
         self,
         cursor: NoneStr = None,
         limit: PositiveInt = 100,
-    ) -> CastsResult:
+    ) -> IterableCastsResult:
         """Get all recent casts
 
         Args:
@@ -831,13 +911,17 @@ class MerkleApiClient:
             limit (PositiveInt, optional): limit, defaults to 100
 
         Returns:
-            CastsResult: model containing casts
+            IterableCastsResult: model containing casts with an optional cursor
         """
-        response = self._get(
-            "recent-casts",
-            params={"cursor": cursor, "limit": limit},
+        response = CastsGetResponse(
+            **self._get(
+                "recent-casts",
+                params={"cursor": cursor, "limit": limit},
+            )
+        )
+        return IterableCastsResult(
+            casts=response.result.casts, cursor=getattr(response.next, "cursor", None)
         )
-        return CastsGetResponse(**response).result
 
     def _recent_casts_lists(
         self,
