@@ -1,7 +1,6 @@
 from typing import List, Optional, Union
 
 from humps import camelize
-from pydantic import AnyUrl
 from pydantic import BaseModel as PydanticBaseModel
 from pydantic import Field, NoneStr, PositiveInt
 
@@ -37,7 +36,7 @@ class ApiToken(BaseModel):
 
 
 class ApiOpenGraphMetadata(BaseModel):
-    url: AnyUrl
+    url: str
     title: NoneStr = None
     description: NoneStr = None
     domain: NoneStr = None
@@ -58,7 +57,7 @@ class ApiOpenSeaNft(BaseModel):
 
 
 class ApiPfp(BaseModel):
-    url: AnyUrl
+    url: str
     verified: bool
 
 
@@ -454,6 +453,11 @@ class EventsResult(BaseModel):
     events: List[ApiAssetEvent]
 
 
+class IterableEventsResult(BaseModel):
+    events: List[ApiAssetEvent]
+    cursor: NoneStr = None
+
+
 class AssetEventsGetResponse(BaseModel):
     result: EventsResult
     next: Optional[Next] = None
@@ -502,6 +506,11 @@ class CastsResult(BaseModel):
     casts: List[ApiCast]
 
 
+class IterableCastsResult(BaseModel):
+    casts: List[ApiCast]
+    cursor: NoneStr = None
+
+
 class CastsGetResponse(BaseModel):
     result: CastsResult
     next: Optional[Next] = None
@@ -514,7 +523,7 @@ class Parent(BaseModel):
 
 class CastsPostRequest(BaseModel):
     text: str
-    embeds: Optional[List[AnyUrl]] = None
+    embeds: Optional[List[str]] = None
     parent: Optional[Parent] = None
 
 
@@ -534,6 +543,11 @@ class ReactionsResult(BaseModel):
     likes: List[ApiCastReaction]
 
 
+class IterableReactionsResult(BaseModel):
+    likes: List[ApiCastReaction]
+    cursor: NoneStr = None
+
+
 class ReactionsPutResult(BaseModel):
     like: ApiCastReaction
 
@@ -563,6 +577,11 @@ class UsersResult(BaseModel):
     users: List[ApiUser]
 
 
+class IterableUsersResult(BaseModel):
+    users: List[ApiUser]
+    cursor: NoneStr = None
+
+
 class CastRecastersGetResponse(BaseModel):
     result: UsersResult
     next: Optional[Next] = None
@@ -572,6 +591,11 @@ class CollectionsResult(BaseModel):
     collections: List[ApiAssetCollection]
 
 
+class IterableCollectionsResult(BaseModel):
+    collections: List[ApiAssetCollection]
+    cursor: NoneStr = None
+
+
 class UserCollectionsGetResponse(BaseModel):
     result: CollectionsResult
     next: Optional[Next] = None
@@ -610,6 +634,11 @@ class Likes(BaseModel):
     likes: List[ApiCastReaction]
 
 
+class IterableLikes(BaseModel):
+    likes: List[ApiCastReaction]
+    cursor: NoneStr = None
+
+
 class UserCastLikesGetResponse(BaseModel):
     result: Likes
     next: Optional[Next] = None
@@ -658,6 +687,11 @@ class NotificationsResult(BaseModel):
     notifications: List[Union[MentionNotification, ReplyNotification]]
 
 
+class IterableNotificationsResult(BaseModel):
+    notifications: List[Union[MentionNotification, ReplyNotification]]
+    cursor: NoneStr = None
+
+
 class MentionAndReplyNotificationsGetResponse(BaseModel):
     result: NotificationsResult
     next: Optional[Next] = None
@@ -679,6 +713,11 @@ class VerificationsResult(BaseModel):
     verifications: List[ApiVerification]
 
 
+class IterableVerificationsResult(BaseModel):
+    verifications: List[ApiVerification]
+    cursor: NoneStr = None
+
+
 class VerificationsGetResponse(BaseModel):
     result: VerificationsResult
     next: Optional[Next] = None
